from cxwidgets.aQt.QtCore import QTimer, Qt, QSize, pyqtSlot, pyqtProperty
from cxwidgets.aQt.QtGui import QColor, QPainter, QRadialGradient, QBrush
from cxwidgets.aQt.QtWidgets import QWidget


class LedWidget(QWidget):
    def __init__(self, parent=None):
        super().__init__(parent)
        self._diamX, self._diamY, self._diameter = 0, 0, 20
        self._color = QColor("red")
        self._alignment = Qt.AlignCenter
        self._state = True
        self._flashing = False
        self._flashRate = 200
        self._timer = QTimer()
        self._timer.timeout.connect(self.toggleState)
        self.setDiameter(self._diameter)

    def paintEvent(self, event):
        painter = QPainter()
        x = 0
        y = 0
        if self._alignment & Qt.AlignLeft:
            x = 0
        elif self._alignment & Qt.AlignRight:
            x = self.width() - self._diameter
        elif self._alignment & Qt.AlignHCenter:
            x = (self.width() - self._diameter) / 2
        elif self._alignment & Qt.AlignJustify:
            x = 0

        if self._alignment & Qt.AlignTop:
            y = 0
        elif self._alignment & Qt.AlignBottom:
            y = self.height() - self._diameter
        elif self._alignment & Qt.AlignVCenter:
            y = (self.height() - self._diameter) / 2

        gradient = QRadialGradient(x + self._diameter / 2, y + self._diameter / 2,
                                   self._diameter * 0.4, self._diameter * 0.4, self._diameter * 0.4)
        gradient.setColorAt(0, Qt.white)

        if self._state:
            gradient.setColorAt(1, self._color)
        else:
            gradient.setColorAt(1, self._color.darker(250))

        painter.begin(self)
        brush = QBrush(gradient)
        painter.setPen(self._color)
        painter.setRenderHint(QPainter.Antialiasing, True)
        painter.setBrush(brush)
        painter.drawEllipse(x, y, self._diameter - 1, self._diameter - 1)

        if self._flashRate > 0 and self._flashing:
            self._timer.start(self._flashRate)
        else:
            self._timer.stop()

        painter.end()

    def minimumSizeHint(self):
        return QSize(self._diameter, self._diameter)

    def sizeHint(self):
        return QSize(self._diameter, self._diameter)

    def getDiameter(self):
        return self._diameter

    @pyqtSlot(int)
    def setDiameter(self, value):
        self._diameter = value
        self.update()

    def getColor(self):
        return self._color

    @pyqtSlot(QColor)
    def setColor(self, value):
        self._color = value
        self.update()

    def getAlignment(self):
        return self._alignment

    @pyqtSlot(Qt.Alignment)
    def setAlignment(self, value):
        self._alignment = value
        self.update()

    def getState(self):
        return self._alignment

    @pyqtSlot(bool)
    def setState(self, value):
        self._state = value
        self.update()

    @pyqtSlot()
    def toggleState(self):
        self._state = not self._state
        self.update()

    def isFlashing(self):
        return self._flashing

    @pyqtSlot(bool)
    def setFlashing(self, value):
        self._flashing = value
        self.update()

    def getFlashRate(self):
        return self._flashRate

    @pyqtSlot(int)
    def setFlashRate(self, value):
        self._flashRate = value
        self.update()

    @pyqtSlot()
    def startFlashing(self):
        self.setFlashing(True)

    @pyqtSlot()
    def stopFlashing(self):
        self.setFlashing(False)

    def flashOnce(self):
        self.toggleState()
        self._timer.singleShot(200, self.toggleState)

    diameter = pyqtProperty(int, getDiameter, setDiameter)
    color = pyqtProperty(QColor, getColor, setColor)
    alignment = pyqtProperty(Qt.Alignment, getAlignment, setAlignment)
    state = pyqtProperty(bool, getState, setState)
    flashing = pyqtProperty(bool, isFlashing, setFlashing)
    flashRate = pyqtProperty(int, getFlashRate, setFlashRate)




#!/usr/bin/env python3

from cxwidgets.aQt import QtWidgets, QtCore
import pycx4.qcda as cda
import pyqtgraph as pg
import numpy as np
import sys
import time

class ChanHist:
    def __init__(self, cname, hist_time):
        self._cname, self._hist_time = cname, hist_time
        self.chan, self.ar_len, self.pos, self.used = None, 4096, 0, 0
        self.ar_inc = 4096

        self.data_v, self.data_t = np.zeros(self.ar_len), np.zeros(self.ar_len)
        self.cx_connect()

    def cx_connect(self):
        if self._cname is None:
            return
        self.chan = cda.DChan(self._cname, private=True, on_update=True)
        self.chan.valueMeasured.connect(self.cs_update)

    def cs_update(self, chan):
        if self.pos == 0:
            if self.used == 0:
                t_min = 0
            else:
                t_min = time.time() - self.data_t[1]
        else:
            t_min = time.time() - self.data_t[0]
        if self.ar_len <= self.pos and t_min < self._hist_time:
            self.ar_len += self.ar_inc
            self.data_v = np.resize(self.data_v, self.ar_len)
            self.data_t = np.resize(self.data_t, self.ar_len)
        if t_min >= self._hist_time:
            self.used = self.pos
            self.pos = 0
        self.data_v[self.pos] = chan.val
        self.data_t[self.pos] = chan.time / 1.e6
        self.pos += 1

    def _get_xy(self):
        cur_t = time.time()
        if self.used > 0:
            vals = np.roll(self.data_v[:self.used], -1 * self.pos)
            times = np.roll(self.data_t[:self.used], -1 * self.pos)
        else:
            vals = np.roll(self.data_v[:self.pos], -1 * self.pos)
            times = np.roll(self.data_t[:self.pos], -1 * self.pos)
        return times-cur_t, vals

    xy = property(_get_xy)


class CXHistPlot(pg.PlotWidget):
    def __init__(self, parent=None, backdround='default', **kwargs):
        super().__init__(parent, backdround, **kwargs)
        if 'cname' in kwargs:
            cname = kwargs['cname']
        else:
            cname = None
        if 'hist_time' in kwargs:
            self._hist_time = kwargs['hist_time']
        else:
            self._hist_time = 100

        self.setXRange(-1 * self._hist_time, 0)

        self.colors = ['#ff0000', '#00ff00', '#0000ff', '#ff00ff']
        self.cnames = []
        self.plts = []
        self.curvs = []
        self.hists = []

        if cname:
            self.addChan(cname)

        self.timer = QtCore.QTimer()
        self.timer.timeout.connect(self.upd_plot)
        self.timer.start(1000)


    def upd_plot(self):
        for ind in range(len(self.cnames)):
            xy = self.hists[ind].xy
            self.curvs[ind].setData(y=xy[1], x=xy[0])

    def addChan(self, cname):
        # check length
        ind = len(self.cnames)
        self.cnames.append(cname)
        self.hists.append(ChanHist(cname, self._hist_time))
        if ind == 0:
            p = self.plotItem
            self.plts.append(p)
            c = self.plot(name=cname, pen=pg.mkPen(self.colors[ind]))
            p.getAxis('left').setLabel(cname, color=self.colors[ind])
            self.curvs.append(c)
            p.vb.sigResized.connect(self.updateviews)
        else:
            p = pg.ViewBox()
            ax = pg.AxisItem('right')
            self.plts[0].layout.addItem(ax, 2, 3+ind)
            self.plts[0].scene().addItem(p)
            ax.linkToView(p)
            p.setXLink(self.plts[0])
            ax.setZValue(-10000)

            ax.setLabel(cname, color=self.colors[ind])
            self.plts.append(p)
            c = pg.PlotCurveItem(name=cname, pen=pg.mkPen(self.colors[ind]))
            p.addItem(c)
            self.curvs.append(c)
        self.updateviews()

    def updateviews(self):
        for p in self.plts[1:]:
            p.setGeometry(self.plts[0].vb.sceneBoundingRect())
            p.linkedViewChanged(self.plts[0].vb, p.XAxis)


class CXHistPlot2(pg.GraphicsLayoutWidget):
    def __init__(self, parent=None, **kwargs):
        super().__init__(parent, **kwargs)
        self.cnames = kwargs.get('cnames', None)
        self._hist_time = kwargs.get('hist_time', 100)
        # self.ci - central item = graphics layout
        # # layout next is a difference
        # l = pg.GraphicsLayout()
        # pw.setCentralWidget(l)

        #self.setXRange(-1 * self._hist_time, 0)

        self.colors = ['#ff0000', '#00ff00', '#0000ff', '#ff00ff']
        self.cnames = []
        self.plts = []
        self.curvs = []
        self.hists = []

        self.axs = []

        # if cname:
        #     self.addChan(cname)

        self.timer = QtCore.QTimer()
        self.timer.timeout.connect(self.upd_plot)
        self.timer.start(1000)

        self.max_chans = 4



    def addChan(self, cname):
        pass

    # Axis
    a2 = pg.AxisItem("left")
    a3 = pg.AxisItem("left")
    a4 = pg.AxisItem("left")
    a5 = pg.AxisItem("left")
    a6 = pg.AxisItem("left")

    # ViewBoxes
    v2 = pg.ViewBox()
    v3 = pg.ViewBox()
    v4 = pg.ViewBox()
    v5 = pg.ViewBox()
    v6 = pg.ViewBox()


    # add axis to layout
    ## watch the col parameter here for the position
    l.addItem(a2, row=2, col=5, rowspan=1, colspan=1)
    l.addItem(a3, row=2, col=4, rowspan=1, colspan=1)
    l.addItem(a4, row=2, col=3, rowspan=1, colspan=1)
    l.addItem(a5, row=2, col=2, rowspan=1, colspan=1)
    l.addItem(a6, row=2, col=1, rowspan=1, colspan=1)

    # plotitem and viewbox
    ## at least one plotitem is used whioch holds its own viewbox and left axis
    pI = pg.PlotItem()
    v1 = pI.vb  # reference to viewbox of the plotitem
    l.addItem(pI, row=2, col=6, rowspan=1, colspan=1)  # add plotitem to layout

    # add viewboxes to layout
    l.scene().addItem(v2)
    l.scene().addItem(v3)
    l.scene().addItem(v4)
    l.scene().addItem(v5)
    l.scene().addItem(v6)

    # link axis with viewboxes
    a2.linkToView(v2)
    a3.linkToView(v3)
    a4.linkToView(v4)
    a5.linkToView(v5)
    a6.linkToView(v6)

    # link viewboxes
    v2.setXLink(v1)
    v3.setXLink(v2)
    v4.setXLink(v3)
    v5.setXLink(v4)
    v6.setXLink(v5)

    # axes labels
    pI.getAxis("left").setLabel('axis 1 in ViewBox of PlotItem', color='#FFFFFF')
    a2.setLabel('axis 2 in Viewbox 2', color='#2E2EFE')
    a3.setLabel('axis 3 in Viewbox 3', color='#2EFEF7')
    a4.setLabel('axis 4 in Viewbox 4', color='#2EFE2E')
    a5.setLabel('axis 5 in Viewbox 5', color='#FFFF00')
    a6.setLabel('axis 6 in Viewbox 6', color='#FE2E64')

    # slot: update view when resized
    def updateViews():
        v2.setGeometry(v1.sceneBoundingRect())
        v3.setGeometry(v1.sceneBoundingRect())
        v4.setGeometry(v1.sceneBoundingRect())
        v5.setGeometry(v1.sceneBoundingRect())
        v6.setGeometry(v1.sceneBoundingRect())





app = QtWidgets.QApplication(sys.argv)


# plt = pg.plot(np.random.normal(size=100), title="Simplest possible plotting example")
# print(plt)
w = CXHistPlot(cname='sled1.Imes', hist_time=5000)
#w.addChan('WG4_2.Imes')

w.show()


sys.exit(app.exec_())



from cxwidgets.aQt.QtCore import pyqtSlot, pyqtProperty, Qt
import pycx4.qcda as cda
from .pdoublespinbox import PDoubleSpinBox


class CXDoubleSpinBox(PDoubleSpinBox):
    def __init__(self, parent=None, **kwargs):
        super().__init__(parent, **kwargs)
        self._cname = kwargs.get('cname', None)
        self.chan = None
        self.cx_connect()
        self.done.connect(self.cs_send)

    def contextMenuEvent(self, event):
        print("menu?")

    def mousePressEvent(self, event):
        if event.button() == Qt.LeftButton:
            print("Left Button Clicked")
        elif event.button() == Qt.RightButton:
            print("Right Button Clicked")


    def cx_connect(self):
        if self._cname is None or self._cname == '':
            return
        self.chan = cda.DChan(self._cname, private=True)
        self.chan.valueChanged.connect(self.cs_update)

    @pyqtSlot(float)
    def cs_send(self, value):
        if value == self.chan.val:
            return
        self.chan.setValue(value)

    def cs_update(self, chan):
        if self.value() == chan.val:
            return
        self.setValue(chan.val)

    @pyqtSlot(str)
    def setCname(self, cname):
        if self._cname == cname:
            return
        self._cname = cname
        self.cx_connect()

    def getCname(self):
        return self._cname

    cname = pyqtProperty(str, getCname, setCname)




from cxwidgets.aQt.QtCore import Qt, pyqtSignal
from cxwidgets.aQt.QtWidgets import QDoubleSpinBox


# class SpinWheelEventFilter(QObject):
#     def eventFilter(self, receiver, event):
#         if event.type() == QEvent.Wheel and receiver.focusPolicy() == Qt.WheelFocus:
#             event.accept()
#             return False
#         else:
#             event.ignore()
#             return True
#         #Call Base Class Method to Continue Normal Event Processing
#         #return super(MyEventFilter,self).eventFilter(receiver, event)


class PDoubleSpinBox(QDoubleSpinBox):
    done = pyqtSignal(float)

    def __init__(self, parent=None, **kwargs):
        super().__init__(parent)
        self.valueChanged.connect(self.done)
        self.setMinimum(kwargs.get('min', -100000.0))
        self.setMaximum(kwargs.get('max', 100000.0))

        self.setFocusPolicy(Qt.StrongFocus)

    def wheelEvent(self, event):
        if self.hasFocus():
            super(PDoubleSpinBox, self).wheelEvent(event)
        else:
            event.ignore()

    def focusInEvent(self, event):
        self.setFocusPolicy(Qt.WheelFocus)
        self.update()

    def focusOutEvent(self, event):
        self.setFocusPolicy(Qt.StrongFocus)
        self.update()



from cxwidgets import BaseGridW, PSpinBox
from cxwidgets.aQt.QtWidgets import QLabel, QLineEdit, QTextEdit
from cxwidgets.aQt.QtCore import Qt


class CXGeneralCM(BaseGridW):
    def __init__(self, source_w=None, parent=None):
        super().__init__(parent)
        self.source_w = source_w
        self.grid.addWidget(QLabel("handle info"), 0, 0, 1, 2, Qt.AlignHCenter)

        self.grid.addWidget(QLabel("name"), 1, 0)
        self.name_line = QLineEdit()
        self.grid.addWidget(self.name_line, 1, 1)
        self.name_line.setText(self.source_w.cname)
        self.name_line.setReadOnly(True)

        self.grid.addWidget(QLabel("flags"), 2, 0, 1, 2, Qt.AlignHCenter)
        self.flags_te = QTextEdit()
        self.grid.addWidget(self.flags_te, 3, 0, 1, 2, Qt.AlignHCenter)
        ft = source_w.chan.rflags_text()
        self.flags_te.setText('\n'.join(ft))



from cxwidgets import BaseGridW, PSpinBox
from cxwidgets.aQt.QtWidgets import QLabel, QLineEdit
from cxwidgets.aQt.QtCore import Qt
from .general_cm import CXGeneralCM


class CXSpinboxCM(CXGeneralCM):
    def __init__(self, source_w=None, parent=None):
        super().__init__(source_w, parent)

        rc = self.grid.rowCount()

        self.grid.addWidget(QLabel("step"), rc + 2, 0)
        self.step_sb = PSpinBox()
        self.grid.addWidget(self.step_sb, rc + 2, 1)
        self.step_sb.setValue(source_w.singleStep())
        self.step_sb.done.connect(source_w.setSingleStep)

        self.grid.addWidget(QLabel("min"), rc + 3, 0)
        self.min_sb = PSpinBox()
        self.grid.addWidget(self.min_sb, rc + 3, 1)
        self.min_sb.setValue(source_w.minimum())
        self.min_sb.done.connect(source_w.setMinimum)

        self.grid.addWidget(QLabel("max"), rc + 4, 0)
        self.max_sb = PSpinBox()
        self.grid.addWidget(self.max_sb, rc + 4, 1)
        self.max_sb.setValue(source_w.maximum())
        self.max_sb.done.connect(source_w.setMaximum)

        source_w.chan.get_range()
        source_w.chan.get_strings()
        print(source_w.chan.quant)


from cxwidgets.aQt.QtWidgets import QLCDNumber
from cxwidgets.aQt.QtCore import pyqtSlot, pyqtProperty
import pycx4.qcda as cda


class CXLCDNumber(QLCDNumber):
    def __init__(self, parent=None, **kwargs):
        super().__init__(parent)
        self._cname = kwargs.get('cname', None)
        self.chan = None
        self.cx_connect()

    def cx_connect(self):
        if self._cname is None or self._cname == '':
            return
        self.chan = cda.DChan(self._cname, private=True)
        self.chan.valueChanged.connect(self.cs_update)

    def cs_update(self, chan):
        self.display(chan.val)

    @pyqtSlot(float)
    def setCname(self, cname):
        self._cname = cname
        self.cx_connect()

    def getCname(self):
        return self._cname

    cname = pyqtProperty(str, getCname, setCname)



from cxwidgets.aQt.QtWidgets import QSpinBox
from cxwidgets.aQt.QtCore import pyqtSignal, Qt

class PSpinBox(QSpinBox):
    done = pyqtSignal(int)

    def __init__(self, parent=None, **kwargs):
        super().__init__(parent)
        self.valueChanged.connect(self.done)
        self.setMinimum(kwargs.get('min', -100000))
        self.setMaximum(kwargs.get('max', 100000))

    def wheelEvent(self, event):
        if self.hasFocus():
            super(PSpinBox, self).wheelEvent(event)
        else:
            event.ignore()

    def focusInEvent(self, event):
        self.setFocusPolicy(Qt.WheelFocus)
        self.update()

    def focusOutEvent(self, event):
        self.setFocusPolicy(Qt.StrongFocus)
        self.update()



from cxwidgets.aQt.QtCore import pyqtProperty, pyqtSlot
import pycx4.qcda as cda
from .pledwidget import LedWidget


class CXEventLed(LedWidget):
    def __init__(self, parent=None, **kwargs):
        super().__init__(parent)
        self._cname = kwargs.get('cname', None)
        self.setState(False)
        self.chan = None
        self.cx_connect()

    def cx_connect(self):
        if self._cname is None or self._cname == '':
            return
        self.chan = cda.IChan(self._cname, private=True, on_update=True)
        self.chan.valueMeasured.connect(self.cs_update)

    def cs_update(self, chan):
        self.flashOnce()

    @pyqtSlot(str)
    def set_cname(self, cname):
        if self._cname == cname:
            return
        self._cname = cname
        self.cx_connect()

    def get_cname(self):
        return self._cname

    cname = pyqtProperty(str, get_cname, set_cname)


class CXStateLed(LedWidget):
    def __init__(self, parent=None, **kwargs):
        super().__init__(parent)
        self._cname = kwargs.get('cname', '')
        self.chan = None
        self.cx_connect()

    def cx_connect(self):
        if self._cname == '':
            return
        self.chan = cda.IChan(self._cname, private=True, on_update=True)
        self.chan.valueMeasured.connect(self.cs_update)

    def cs_update(self, chan):
        self.setState(bool(chan.val))

    @pyqtSlot(str)
    def set_cname(self, cname):
        if self._cname == cname:
            return
        self._cname = cname
        self.cx_connect()

    def get_cname(self):
        return self._cname

    cname = pyqtProperty(str, get_cname, set_cname)


